# -*- coding: utf-8 -*-

__author__	= 'Souheil CHELFOUH <souheil@chelfouh.com>'
__docformat__	= 'plaintext'


from sets import Set
from StringIO import StringIO

from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.migrate import migrateFTIs

from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin

from Products.StructuredDocument.config import *
from Products.StructuredDocument.Extensions.utils import *


def install(self):
    """
    Install StructuredDocument
    """
    out = StringIO()

    # Install types
    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)


    # Install skin
    install_subskin(self, out, product_globals)
    registerResources(self, out, 'portal_css', STYLESHEETS)
    registerResources(self, out, 'portal_javascripts', JAVASCRIPTS)


    # Add index
    pc = getToolByName(self, 'portal_catalog')
    if 'object_implements' not in pc.indexes():
        pc.addIndex('object_implements', 'KeywordIndex')
        out.write('Added object_implements index to portal_catalog\n')


    # Migrate FTI, to make sure we get the necessary infrastructure for the
    # 'display' menu to work.
    migrated = migrateFTIs(self, product=PROJECTNAME)
    

    # Enable portal_factory
    if len(FACTORY):
        
        factory = getToolByName(self, 'portal_factory')
        types = factory.getFactoryTypes().keys()
        
        for myType in FACTORY:
            if myType not in types:
                types.append(myType)
                
        factory.manage_setPortalFactoryTypes(listOfTypeIds = types)


    # Hide types to navigation    
    portalProperties = getToolByName(self, 'portal_properties')
    navtreeProps = getattr(portalProperties, 'navtree_properties')
    siteProps = getattr(portalProperties, 'site_properties')

    if HIDDEN:
        addToListProperty(self, out, navtreeProps,
                          'metaTypesNotToList', HIDDEN)
        addToListProperty(self, out, siteProps,
                          'types_not_searched', HIDDEN)
        addToListProperty(self, out, navtreeProps,
                          'parentMetaTypesNotToQuery', HIDDEN)


    # Add to default_page_types
    addToListProperty(self, out, siteProps,
                      'default_page_types', ('StructuredDocument',))
    

    # Add action
    at = getToolByName(self, 'portal_actions')
    ai = getToolByName(self, 'portal_actionicons')
    try:
        at.addAction('structured-document-toggle-edit',
                     name='Toggle edit mode',
                     action='',
                     condition="python: object.portal_type in ('StructuredDocument', 'SimpleChapter')",
                     permission='Modify portal content',
                     category='document_actions')

        ai.addActionIcon('plone',
                     'structured-document-toggle-edit',
                     'edit_mode.gif',
                     'Toggle edit mode', 0)
    except:
        pass


    # Adding kupu configuration for our types
    kupuTool    = getToolByName(self, 'kupu_library_tool')
    linkable    = Set(kupuTool.getPortalTypesForResourceType('linkable'))
    mediaobject = Set(kupuTool.getPortalTypesForResourceType('mediaobject'))
    collection  = Set(kupuTool.getPortalTypesForResourceType('collection'))

    [linkable.add(link) for link in LINKABLE]
    [mediaobject.add(media) for media in MEDIAOBJECT]
    [collection.add(item) for item in COLLECTION]
    
    kupuTool.updateResourceTypes(({'resource_type' : 'linkable',
                                   'old_type'      : 'linkable',
                                   'portal_types'  :  linkable},
                                  {'resource_type' : 'mediaobject',
                                   'old_type'      : 'mediaobject',
                                   'portal_types'  :  mediaobject},
                                  {'resource_type' : 'collection',
                                   'old_type'      : 'collection',
                                   'portal_types'  :  collection},))

    out.write("StructuredDocument's kupu configuration is done.")
    out.write("StructuredDocument is installed.")
    return out.getvalue()


def uninstall(self):
    out = StringIO()

    resetResources(self, out, 'portal_css', STYLESHEETS)
    resetResources(self, out, 'portal_javascripts', JAVASCRIPTS)

    return out.getvalue()
