# -*- coding: utf-8 -*-

from zope.interface import implements
from Products.StructuredDocument.interfaces import INextPreviousProvider
from Products.CMFCore.utils import getToolByName


class OrderedFolderNextPrevious( object ):
    """ Adapts next-previous for an Ordered Folder """

    implements( INextPreviousProvider )

    def __init__(self, context):
        self.context   = context


    def getNextItem(self, oid):
        relatives = self.itemRelatives(oid)
        return relatives["next"]
        

    def getPreviousItem(self, oid):
        relatives = self.itemRelatives(oid)
        return relatives["previous"]


    def buildNextPreviousQuery(self, position, range, sort_order = None):
        """
        """
        sort_on = 'getObjPositionInParent'
        query   = dict(
            sort_on    = sort_on,
            object_implements = 'Products.StructuredDocument.interfaces.chapter.IStructuredChapter',
            sort_limit = 1,
            path       = dict(query = '/'.join(self.context.getPhysicalPath()),
                              depth = 1)
            )
                
        # Query the position using a range
        query[sort_on] = position and dict(query = position, range = range) or 0

        # Filters on content
        query['sort_order'] = sort_order or 'ascending'
        query['is_default_page'] = False

        return query


    def buildNextPreviousItem(self, brain):
        """
        """
        item = {
            'id': brain.id,
            'url': self.getViewUrl(brain),
            'title': brain.Title,
            'description': brain.Description,
            'contentType': brain.portal_type,
            }
        return item


    def getViewUrl(self, brain):
        """create link and support contents that requires /view 
        """
        props = getToolByName(self.context, 'portal_properties')       
        view_action_types = props.site_properties.getProperty('typesUseViewActionInListings', ())
    
        if brain.portal_type in view_action_types:
            return brain.getURL() + '/view'
    
        return brain.getURL()


    def itemRelatives(self, oid):
        """
        """
        folder   = self.context
        catalog  = getToolByName(self.context, 'portal_catalog')
        position = folder.getObjectPosition(oid)

        previous = None
        next     = None

        # Get the previous item
        if position - 1 >= 0:
            prev_brain = catalog(self.buildNextPreviousQuery(position   = position - 1,
                                                             range      = 'max',
                                                             sort_order = 'reverse'))
            if prev_brain and len(prev_brain):
                previous = self.buildNextPreviousItem(prev_brain[0])


        # Get the next item
        if (position + 1) < len(folder._objects):
            next_brain = catalog(self.buildNextPreviousQuery(position   = position + 1,
                                                             range      = 'min'))
            
            if next_brain and len(next_brain):
                next   = self.buildNextPreviousItem(next_brain[0])


        nextPrevious = {
            'next'      : next,
            'previous'  : previous,
            }

        return nextPrevious

